/**++
 *   
 *   LICENSE
 *   -------
 *   
 *   Copyright (c) 2004 Renato Mancuso
 *   All rights reserved.
 *   
 *   Redistribution and use in source and binary forms, with or without modification, are 
 *   permitted provided that the following conditions are met:
 *   
 *   - Redistributions of source code must retain the above copyright notice, this list 
 *     of conditions and the following disclaimer.
 *   
 *   - Redistributions in binary form must reproduce the above copyright notice, this list
 *     of conditions and the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   
 *   - Neither the name of Renato Mancuso nor the names of its contributors may be used to 
 *     endorse or promote products derived from this software without specific prior written 
 *     permission.
 *   
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS AS IS AND ANY EXPRESS 
 *   OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 *   AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 *   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 *   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 *   IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 *   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *   
--**/


#include "FileParserHost.hpp"
#include "ParserErrors.hpp"

#include <sstream>

namespace OpenEMBL
{
namespace Phoenix
{

    FileParserHost::FileParserHost(char const* theFilename)
        : m_TextSource(NULL)
        , m_Filename(theFilename)
    {
    }

    FileParserHost::~FileParserHost() throw()
    {
        delete m_TextSource;
    }

    ITextSource* FileParserHost::getTextSource()
    {
        if (NULL == m_TextSource)
            m_TextSource = new TextFile(m_Filename.c_str());

        return m_TextSource;
    }

    LPCSTR FileParserHost::getSourceDisplayName()
    {
        return m_Filename.c_str();
    }

    METHODIMP FileParserHost::parse()
    {
        try
        {
            ParserHost::parse();
        }
        catch (OSError& e)
        {
            onError(m_TextSource != NULL ? m_TextSource->currentLineNumber() : 0,
                    ERROR_OS_ERROR,
                    e.getMessage());
        }
        catch (LineTooLong& e)
        {
            onError(e.getLineNumber(),
                    ERROR_LINE_TOO_LONG,
                    NULL);
        }    
    }

} 
}
