/**++
 *   
 * 2006-09-15 Christian Quast (CQ) - adaptation to the new EMBL (r87+) ID line format
 *
 *
 *   LICENSE
 *   -------
 *   
 *   Copyright (c) 2004 Renato Mancuso
 *   All rights reserved.
 *   
 *   Redistribution and use in source and binary forms, with or without modification, are 
 *   permitted provided that the following conditions are met:
 *   
 *   - Redistributions of source code must retain the above copyright notice, this list 
 *     of conditions and the following disclaimer.
 *   
 *   - Redistributions in binary form must reproduce the above copyright notice, this list
 *     of conditions and the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   
 *   - Neither the name of Renato Mancuso nor the names of its contributors may be used to 
 *     endorse or promote products derived from this software without specific prior written 
 *     permission.
 *   
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS AS IS AND ANY EXPRESS 
 *   OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
 *   AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 *   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 *   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
 *   IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 *   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *   
--**/


#ifndef OPEN_EMBL_ID_LINE_PARSER_INCLUDED
#define OPEN_EMBL_ID_LINE_PARSER_INCLUDED

#if !defined(OPEN_EMBL_NO_PRAGMA_ONCE)
#pragma once
#endif

#include "ItemParserImpl.hpp"
#include "FieldDefs.hpp"

namespace OpenEMBL
{
namespace Phoenix
{

class IDLineParser
            : public ItemParserImpl<IDLine>
{
public:
    IDLineParser(
        IParserCtx*             pParserCtx,
        IItemHandler<IDLine>*   pLineHandler);

    METHOD parse();

private:
    void processLine(const std::string & theLine, long theLineNumber);

    bool getEntrynameDataclass  (String const & theToken, long theLineNumber, IDLine & theField);
    bool getMoltype             (String const & theToken, long theLineNumber, IDLine & theField);
    bool getDivision            (String const & theToken, long theLineNumber, IDLine & theField);
    bool getSequenceLength      (String const & theToken, long theLineNumber, IDLine & theField);

    //*** added by CQ - 200609015
    bool getEntryName           (String const & theToken, long theLineNumber, IDLine & theField);
    bool getDataClass           (String const & theToken, long theLineNumber, IDLine & theField);
    bool getSequenceVersion     (String const & theToken, long theLineNumber, IDLine & theField);
    bool getTopology            (String const & theToken, long theLineNumber, IDLine & theField);
    //***
};

}
}

#endif // OPEN_EMBL_ID_LINE_PARSER_INCLUDED
